

Application create: #MaterialCluster with:
        (#( EtTools FBaseCluster)
            collect: [:each | Smalltalk at: each ifAbsent: [
                self error: 'Not all of the prerequisites are loaded']])!

MaterialCluster becomeDefault!
Object subclass: #AspectDifferences
    instanceVariableNames: 'materialClass aspectClass materialOnlyMethods aspectOnlyMethods '
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
Application subclass: #MaterialCluster
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
Clause subclass: #AspectsClause
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
TmObject subclass: #Material
    instanceVariableNames: ''
    classVariableNames: 'Aspects '
    poolDictionaries: ''!

Material comment: 'Class Variables:
	Aspects:	holds a Dictionary of the aspects lists of all derived material classes.'!

MaterialCluster becomeDefault!
Material subclass: #Aspect
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: 'CwConstants '!

MaterialCluster becomeDefault!
Aspect subclass: #CompositeAspect
    instanceVariableNames: ''
    classVariableNames: 'BaseAspects '
    poolDictionaries: ''!

MaterialCluster becomeDefault!
CompositeAspect subclass: #SimpleBrowsable
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
SimpleBrowsable subclass: #SimpleBrowsableWrapper
    instanceVariableNames: 'wrappedObject '
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
Aspect subclass: #SimpleAspect
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
SimpleAspect subclass: #Indexable
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
Indexable subclass: #IndexableWrapper
    instanceVariableNames: 'wrappedObject '
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
SimpleAspect subclass: #Listable
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
Listable subclass: #ListableWrapper
    instanceVariableNames: 'wrappedObject listNameBlock '
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
SimpleAspect subclass: #SimpleEditable
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
SimpleEditable subclass: #SimpleEditableWrapper
    instanceVariableNames: 'wrappedObject getTextBlock setTextBlock '
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
SimpleAspect subclass: #SimpleViewable
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
SimpleViewable subclass: #SimpleViewableWrapper
    instanceVariableNames: 'wrappedObject getTextBlock '
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
Material subclass: #Document
    instanceVariableNames: 'title '
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
Document subclass: #SimpleDocument
    instanceVariableNames: 'text '
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!
Material subclass: #Folder
    instanceVariableNames: 'title toc '
    classVariableNames: ''
    poolDictionaries: ''!

MaterialCluster becomeDefault!

!Aspect class publicMethods !

addBaseAspectClass: anAspect
	"Default implementation does nothing."!

addToMaterial: material
	"Copy the methods of the aspect to the material class"

	self subclassResponsibility.

	!

allBaseAspectClasses

	^#().
	
!

aspectMethods
	"Returns all methods (compiled methods) which build the
	protocoll of the aspect. These methods should be in a method category
	with the same name as the class (its not tested)"

	| methods |

	methods := self info selectorsArray.
	methods := methods collect: [ :m | self methodDictionary at: m ].
	^methods.
!

baseAspectClasses

	^#().!

baseAspects

	^#().!

deleteMethod: aMethod
	"Delete aMethod from the class."

	EtTools managerInterface removeMethod: aMethod.
!

hasBaseAspect: anAspect

	^false.!

isSimple
	"Is it a SimpleAspect or does it have base aspect classes"

	self subclassResponsibility.!

removeBaseAspectClass: anAspect
	"Default implementation does nothing."!

synchronizeMaterial: materialClass
	"Synchronize the aspect in the material class with this aspect."

	| differences prompter |

	differences := AspectDifferences 
		differencesInAspect: self
		Material: materialClass.

	differences aspectOnlyMethods do: [ :method |
		self copyMethod: method To: materialClass ].		

	differences materialOnlyMethods notEmpty ifTrue: [
		prompter := CwListPrompter new.
		prompter items: (differences materialOnlyMethods collect: [ :m | m selector asString ]);
			selectedItems: prompter items first;
			messageString: 'Delete following methods from ', materialClass printString, '?';
			title: 'Please confirm';
			button1Name: '  Yes  ';
			button2Name: '   No   '.
		prompter prompt notNil ifTrue: [
			differences materialOnlyMethods do: [ :method |
				EtTools managerInterface removeMethod: method. ]]].
! !

!Aspect class privateMethods !

copyMethod: aCM To: materialClass

	aCM methodClass == materialClass ifFalse: [ "skip if it is there"
		(materialClass
			copy: aCM selector
			from: aCM methodClass
			ifNewAddTo: aCM application) isNil ifTrue: [
				System message: aCM printString,' was not copied. Check the source for errors']].
!

copyMethodsTo: materialClass

	| methods materialMethods implementedMethods newMethods overwriteMethods prompter |

	methods := self aspectMethods.
	methods notNil 
		ifTrue: [
			materialMethods := materialClass info selectorsArray.
			implementedMethods := methods select: [ :method |
				materialMethods includes: method selector ].
			newMethods := methods reject: [ :method |
				materialMethods includes: method selector ].

			"If there are some methods allready implemented in the material
			class, notify the user that they are not copied."
			implementedMethods notEmpty ifTrue: [
				prompter := CwListPrompter new.
				prompter items: ((implementedMethods collect: [ :method |
					method selector asString ]) asSortedCollection);
				selectedItems: prompter items first; 
				button2Name: ' OK ';
				messageString: 'Following methods are not copied. They are allready in the material:';
				title: 'Information'.
			"	selectionPolicy: XmEXTENDEDSELECT.  "
				prompter prompt.
			].

			"Copy all methods that are not allready in the material class."
			newMethods do: [ :aCM |
				self copyMethod: aCM To: materialClass. ]].! !


!AspectDifferences class publicMethods !

differencesInAspect: anAspect Material: aMaterial

	| differences |

	differences := self new.
	differences aspectClass: anAspect.
	differences materialClass: aMaterial.
	differences calculateDifferences.
	^differences.
!

new

	^super new initialize.! !


!AspectDifferences publicMethods !

aspectClass

	^aspectClass.
!

aspectClass: aClass

	aspectClass := aClass.
!

aspectOnlyMethods

	^aspectOnlyMethods.
!

aspectOnlyMethods: aCollection

	aspectOnlyMethods := aCollection.
!

calculateDifferences

	| aspectName aspectMethods aspectSelectors materialMethods materialSelectors |

	aspectName := self aspectClass printString.
	aspectMethods := self aspectClass aspectMethods.
	aspectSelectors := self aspectClass info selectorsIn: aspectName.
	materialMethods := self materialClass aspectMethods: self aspectClass.
	materialSelectors := materialClass info selectorsIn: aspectName.

	"Check if there are methods missing in the material class"
	aspectMethods do: [ :cm |
		(materialSelectors includes: cm selector) ifFalse: [
			self aspectOnlyMethods add: cm ]].

	"Check if the material class has methods, which are not methods of the aspect"
	materialMethods do: [ :cm |
		(aspectSelectors includes: cm selector) ifFalse: [
			self materialOnlyMethods add: cm ]].
!

materialClass

	^materialClass.
!

materialClass: aClass

	materialClass := aClass.
!

materialOnlyMethods

	^materialOnlyMethods.
!

materialOnlyMethods: aCollection

	materialOnlyMethods := aCollection.
! !

!AspectDifferences privateMethods !

initialize
	
	self materialOnlyMethods: OrderedCollection new.
	self aspectOnlyMethods: OrderedCollection new.
! !


!AspectsClause class publicMethods !

isAbstract
	^false!

root
	^Material! !


!AspectsClause publicMethods !

adaptToClass: aMaterialClass!

matches: aClause
	^ ( super matches: aClause )! !


!CompositeAspect class publicMethods !

addBaseAspectClass: anAspect
	"Add anAspect as a base aspect."

	(self baseAspects includes: anAspect name)
		ifFalse: [
			self baseAspects add: anAspect name. ].
!

addToMaterial: materialClass
	"Copy the methods of the aspect and all the base aspects
	 to the material class"

	| baseAspects |

	(materialClass hasAspect: self)
		ifFalse: [
			materialClass addAspectClass: self. ].

	(materialClass hasAspectCategory: self)
		ifFalse: [
			self copyMethodsTo: materialClass.
			baseAspects := self baseAspectClasses.
			baseAspects do: [ :aspect |
				aspect addToMaterial: materialClass. ]].
!

allBaseAspectClasses
	"Returns alle base aspect classes with their base aspect classes."
	
	| baseAspects |

	baseAspects := self baseAspectClasses.
	baseAspects notEmpty 
		ifTrue: [
			baseAspects do: [ :aspect |
				baseAspects addAll: aspect allBaseAspectClasses. ]].
	^baseAspects.
!

baseAspectClasses

	| baseAspects |

	baseAspects := self baseAspects.
	baseAspects := baseAspects collect: [ :aspect | Smalltalk classAt: aspect ].
	^baseAspects.
!

baseAspects
	"Gets the base aspects list for the concrete aspect class from the 
	BaseAspects class variable. "

	BaseAspects isNil ifTrue: [
		BaseAspects := Dictionary new ].

	(BaseAspects includesKey: self name) ifFalse: [
		BaseAspects at: self name put: OrderedCollection new ].
	^BaseAspects at: self name.
!

baseAspects: aspectsList
	"Sets the list of base aspects. 
	Used only for administrativ purposes."

	BaseAspects isNil ifTrue: [
		BaseAspects := Dictionary new ].

	BaseAspects at: self name put: aspectsList.
!

hasBaseAspect: anAspect

	| hasAspect |

	hasAspect := self baseAspects includes: anAspect name.
	hasAspect ifFalse: [
		self baseAspectClasses do: [ :aspect |
			(aspect hasBaseAspect: anAspect)
				ifTrue: [ ^true ]]].
	^hasAspect.
!

isSimple
	"Is it a SimpleAspect or does it have base aspect classes"

	^false.
!

removeBaseAspectClass: anAspect
	"Remove anAspect from the base aspects."

	self baseAspects 
		remove: anAspect name
		ifAbsent: [ System message: 'Cannot remove base aspect from aspect who does not have this base aspect.' ].
! !


!Document class publicMethods !

new: docTitle
	^super new initialize: docTitle! !


!Document publicMethods !

<= aListable
	^( self listName ) <= ( aListable listName ).!

docTitle
	^title!

initialize: aTitle
	title := aTitle!

listName
	^self docTitle! !


!Folder class publicMethods !

new: aTitle
	^super new initialize: aTitle! !


!Folder publicMethods !

<= aListable
	^ ( self listName ) <= ( aListable listName )	!

add: aListable
	self toc add: aListable!

asCollection
	^self toc!

at: index
	^self toc at: index!

collect: aBlock
	toc collect: aBlock.!

do: aBlock
	toc do: aBlock.!

initialize: aString

	title := aString.

	toc := SortedCollection new.!

listName
	^self title!

remove: aListable
	self toc remove: aListable!

title
	^title!

toc
	^toc! !


!Indexable publicMethods !

add: anObject
	self subclassResponsibility!

asCollection
	self subclassResponsibility!

at: index
	self subclassResponsibility!

at: index put: aListable
	self subclassResponsibility!

collect: aBlock
	self subclassResponsibility!

do: aBlock
	self subclassResponsibility!

remove: aListable
	self subclassResponsibility!

select: aBlock
	self subclassResponsibility! !


!IndexableWrapper class publicMethods !

new: wrappedObject
	^super new initialize: wrappedObject! !


!IndexableWrapper publicMethods !

add: anObject
	self wrappedObject add: anObject!

asCollection
	^self wrappedObject!

at: index
	^self wrappedObject at: index!

at: index put: aListable
	^self wrappedObject at: index put: aListable!

collect: aBlock
	^self class new: ( self wrappedObject collect: aBlock ).!

do: aBlock
	^self wrappedObject do: aBlock!

initialize: anOrdCol
	wrappedObject := anOrdCol!

remove: anObject
	self wrappedObject remove: anObject!

select: aBlock
	^self class new: ( self wrappedObject do: aBlock )!

wrappedObject
	^wrappedObject! !


!Listable publicMethods !

<= aListable
	^( self listName ) <= ( aListable listName ).!

listName
	self subclassResponsibility! !


!ListableWrapper class publicMethods !

new: wrappedObject
	^super new initialize: wrappedObject! !


!ListableWrapper publicMethods !

initialize
	self listNameBlock: [ :obj |
		obj printString ]!

initialize: anObject
	self initialize.
	wrappedObject := anObject.!

listName
	^self listNameBlock value: ( self wrappedObject )!

listNameBlock
	^listNameBlock
!

listNameBlock: aBlock
	listNameBlock := aBlock!

wrappedObject
	^wrappedObject! !


!Material class publicMethods !

addAspectClass: anAspect
	"Add the name of the aspect class to the aspects class variable"

	self aspects add: anAspect name.
!

allAspectClasses
	"Returns all aspect classes with all their base aspect classes."
	
	| aspects |

	aspects := self aspectClasses.
	aspects notEmpty
		ifTrue: [
			aspects do: [ :aspect |
				aspects addAll: aspect allBaseAspectClasses. ]].
	^aspects.
!

aspectClasses

	| aspects |

	aspects := self aspects.
	aspects := aspects select: [ :aspect | (Smalltalk classAt: aspect) notNil ].
	aspects := aspects collect: [ :aspect | Smalltalk classAt: aspect ].

	^aspects.
!

aspectMethods: anAspect

	| aspectSelectors materialSelectors selectors methods |

	"Look at the aspect class to get all the method selectors"
	aspectSelectors := anAspect aspectMethods collect: [ :cm | cm selector ].
	materialSelectors := self info selectorsArray.
	selectors := aspectSelectors select: [ :selector | materialSelectors includes: selector ].
	methods := selectors collect: [ :m | self methodDictionary at: m ].
	^methods.
!

copyMethod: aCM To: aspectClass

	aCM methodClass == aspectClass ifFalse: [  "skip it if its there"
		(aspectClass
			copy: aCM selector
			from: aCM methodClass
			ifNewAddTo: aCM application) isNil ifTrue: [
				System message: aCM printString,' was not copied. Check the source for errors']].
!

hasAspect: anAspect
	"Tests if the material has the aspect anAspect"

	| hasAspect |

	hasAspect := self aspects includes: anAspect name.
	hasAspect ifFalse: [
		self aspectClasses do: [ :aspect |
			(aspect hasBaseAspect: anAspect)
				ifTrue: [ ^true ]]].
	^hasAspect.

!

hasAspectCategory: anAspect
	"Tests if the material has a method category with the name 
	of the aspect anAspect"

	^self info categories includes: anAspect name asString.
!

hasAspectInSuperclass: anAspect
	"Tests if a superclass of the material has the aspect anAspect"

	| hasAspect material |

	hasAspect := false.
	material := self superclass.
	[(material notNil) & (material ~= Material) & (hasAspect = false)] whileTrue: [
		hasAspect := material hasAspect: anAspect.
		material := material superclass.	].
	^hasAspect.
!

materialsContaining: anAspect
	"Returns a collection of the materials, who
	contain the aspect anAspect."

	| materials aspectHolders |

	materials := Material allSubclasses.
	aspectHolders := materials select: [ :material |
		material hasAspect: anAspect ].

	^aspectHolders.
!

removeAspect: anAspect
	"Removes the aspect and all of it's base aspects from the material."

	| aspects remainingAspects baseAspectClasses |

	"Remove this aspect from the list of aspects and test, if the remaining aspects
	contain this aspect as one of their base aspects. If not, the aspect can be
	removed completely. Else, put the aspect back into the list of aspects."
	(self removeAspectClass: anAspect) ifFalse: [ ^false ].
	remainingAspects := self allAspectClasses.
	(remainingAspects includes: anAspect)
		ifTrue: [
			self addAspectClass: anAspect.
			System message: 'The aspect ', anAspect name asString, ' is part of another aspect. It can not be deleted.'.
			^true. ]
		ifFalse: [
			self removeAspectMethods: anAspect.

			"Remove all base aspects, if they are not part of another aspect."
			baseAspectClasses := anAspect allBaseAspectClasses. 
			baseAspectClasses do: [ :aspect |
			(remainingAspects includes: aspect)
				ifFalse: [
					self removeAspectMethods: aspect. ]]].
	^true.!

removeAspectClass: anAspect
	"Removes the aspect from the list of aspects of the material."

	self aspects 
		remove: anAspect name
		ifAbsent: [ ^false ].
	^true.! !

!Material class privateMethods !

aspects
	"Gets the aspects list for the concrete material class from the Aspects
	class variable. 
	NOTE: The result is a list of aspect names, not aspect classes!!"

	| aspects |

	Aspects isNil ifTrue: [
		Aspects := Dictionary new ]	.

	(Aspects includesKey: self name) ifFalse: [
		Aspects at: self name put: OrderedCollection new ].
	aspects := Aspects at: self name.
	^aspects.
!

initializeAfterLoad
	"Initializes the Aspects dictionary."
		
	super initializeAfterLoad.

	Aspects isNil ifTrue: [
		Aspects := Dictionary new ].!

removeAspectMethods: anAspect
	"Removes the category and all methods of the aspect."

	| aspectName methods |

	"Delete the methods"
	methods := self aspectMethods: anAspect.
	methods do: [ :method |
		EtTools managerInterface removeMethod: method. ].

	"Delete the category corresponding to the aspect."
	aspectName := anAspect name asString.
	(self info hasCategory: aspectName) 
		ifTrue: [ self removeCategory: aspectName. ].! !


Material initializeAfterLoad!

!SimpleAspect class publicMethods !

addToMaterial: materialClass
	"Copy the methods of the aspect to the material class"

	(materialClass hasAspectCategory: self)
		ifFalse: [ self copyMethodsTo: materialClass. ].

	(materialClass hasAspect: self)
		ifFalse: [
			materialClass addAspectClass: self. ].!

isSimple
	"Is it a SimpleAspect or does it have base aspect classes"

	^true.
! !


!SimpleBrowsableWrapper publicMethods !

initialize
!

initialize: anObject
	self initialize.
	wrappedObject := anObject.!

wrappedObject
	^wrappedObject! !


!SimpleDocument publicMethods !

initialize: aTitle

	super initialize: aTitle.
	self text: ''.!

text
	^text!

text: aString
	text := aString! !


!SimpleEditable publicMethods !

text
	self subclassResponsibility!

text: aString
	self subclassResponsibility! !


!SimpleEditableWrapper class publicMethods !

new: getBlock with: setBlock
	^super new initialize: getBlock with: setBlock! !


!SimpleEditableWrapper publicMethods !

getTextBlock
	^getTextBlock!

getTextBlock: aBlock
	getTextBlock := aBlock!

initialize
	self getTextBlock: [ :obj | obj printString ].
	self setTextBlock: [ :obj :text | ]!

initialize: anObject
	self initialize.
	wrappedObject := anObject!

setTextBlock
	^setTextBlock!

setTextBlock: aBlock
	setTextBlock := aBlock!

text
	^ self getTextBlock value: ( self wrappedObject ).
	!

text: aString
	self setTextBlock value: ( self wrappedObject ) value: aString
	!

wrappedObject
	^wrappedObject
! !


!SimpleViewableWrapper class publicMethods !

new: getBlock
	^super new initialize: getBlock! !


!SimpleViewableWrapper publicMethods !

getTextBlock
	^getTextBlock!

getTextBlock: aBlock
	getTextBlock := aBlock.!

initialize
	self getTextBlock: [ :obj |
		obj printString ]!

initialize: anObject
	self initialize.
	wrappedObject := anObject.!

text
	^ self getTextBlock value: ( self wrappedObject ).!

wrappedObject
	^wrappedObject! !

